import java.util.Iterator;

public class BinarySearchTree<K extends Comparable<? super K>, V> implements Iterable {

	private TreeNode<K, V> root;
	
	public BinarySearchTree() {
		super();
		this.root = null;
	}

	public BinarySearchTree(TreeNode<K, V> root) {
		super();
		this.root = root;
	}

	public TreeNode<K, V> getRoot() {
		return root;
	}

	public void setRoot(TreeNode<K, V> root) {
		this.root = root;
	}
	
	public boolean isEmpty() {
		return (this.root == null);
	}
	
	public TreeItem<K, V> getRootItem() throws TreeException {
		if (this.root == null) {
			throw new TreeException("TreeException: BinarySearchTree is empty, no TreeItem available");
		} else {
			return this.root.getTreeItem();
		}
	}

	public void insert(TreeItem<K, V> treeItem) {
		this.insertItem(treeItem);
	}
	
	public TreeItem<K, V> find(K key) {
		return this.findItem(key);
	}
	
	public void delete(K key) {
		TreeNode<K, V> nodeToDelete = this.findNode(key);
		if (nodeToDelete != null) {
			this.deleteNode(nodeToDelete);
		}
		
	}

//	public void insert(TreeItem<K, V> treeItem) {
//		this.root = this.insertItem(this.root, null, treeItem);
//	}
//	
//	public TreeItem<K, V> find(K key) {
//		return this.findItem(this.root, key);
//	}
//
//	public void delete(K key) {
//		this.root = this.deleteItem(this.root, key);
//	}
	
	
	
	@Override
	public Iterator iterator() {
		return new TreeIterator<K, V>(this);
	}
	
	/****************************************************************/
	/*            Private Methods									*/
	/****************************************************************/
//	
//	private TreeNode<K, V> insertItem(TreeNode<K, V> node,
//									TreeNode<K, V> parent,
//									TreeItem<K, V> item) {
//		
//		if (node == null) {
//			node = new TreeNode<K, V>(item);
//			node.setParent(parent);
//		} if (this.compareKeys(node.getTreeItem().getKey(), item.getKey()) > 0) {
//			node.setLeftChild(insertItem(node.getLeftChild(), node, item));
//		} else {
//			node.setRightChild(insertItem(node.getRightChild(), node, item));
//		}
//		
//		return node;
//	}
//	
//	private TreeItem<K, V> findItem(TreeNode<K, V> node, K key) {
//		if (node == null) {
//			return null;
//		} else if (this.compareKeys(node.getTreeItem().getKey(), key) == 0) {
//			return node.getTreeItem();
//		} else if (this.compareKeys(node.getTreeItem().getKey(), key) > 0) {
//			return findItem(node.getLeftChild(), key);
//		} else {
//			return findItem(node.getRightChild(), key);
//		}
//		
//	}
//	
//	private TreeNode<K, V> deleteItem(TreeNode<K, V> node, K key) throws TreeException {
//		if (node == null) {
//			throw new TreeException("TreeException: Item to delete was not found in the tree");
//		} else {
//			TreeItem<K, V> item = node.getTreeItem();
//			if (this.compareKeys(item.getKey(), key) == 0) {
//				node = deleteNode(node);
//			} else if (this.compareKeys(item.getKey(), key) > 0) {
//				node.setLeftChild(deleteItem(node.getLeftChild(), key));
//				if (node.getLeftChild() != null ) {
//					node.getLeftChild().setParent(node);
//				}
//			} else {
//				node.setRightChild(deleteItem(node.getRightChild(), key));
//				if (node.getRightChild() != null) {
//					node.getRightChild().setParent(node);
//				}
//			}
//		}
//		 
//		return node;
//	}
//	
//	private TreeNode<K, V> deleteNode(TreeNode<K, V> node) {
//		if (this.isLeafNode(node)) {
//			return null;
//		} else if (this.hasNoLeftChild(node)) {
//			return node.getRightChild(); 
//		} else if (this.hasNoRightChild(node)) {
//			return node.getLeftChild();
//		} else {
//			TreeNode<K, V> successor = this.findSuccessor (node.getRightChild());
//			node.setTreeItem(successor.getTreeItem());
//			node.setRightChild(this.deleteSuccessorNode(node.getRightChild()));
//			return node;
//		}
//	} 
//	
//	private TreeNode<K, V> findSuccessor(TreeNode<K, V> node) {
//		if (node.getLeftChild() == null) {
//			return node;
//		} else {
//			return findSuccessor(node.getLeftChild());
//		}
//	}
//	
//	private TreeNode<K, V> deleteSuccessorNode(TreeNode<K, V> node) {
//		if (node.getLeftChild() == null) {
//			return node.getRightChild();
//		} else {
//			node.setLeftChild(this.deleteSuccessorNode(node.getLeftChild()));
//			return node;
//		}
//	}
	
	private void insertItem(TreeItem<K, V> treeItem) {
		TreeNode<K, V> newNode = new TreeNode<K, V>(treeItem);
		
		if (this.root == null) {
			this.root = newNode;
			this.root.setParent(null);
		} else {
			TreeNode<K, V> curNode = root;
			while (curNode != null) {
				if (this.compareKeys(curNode.getTreeItem().getKey(), treeItem.getKey()) > 0) {
					// treeItem.key < curNode.key -----> go Left
					if (curNode.getLeftChild() == null) {
						curNode.setLeftChild(newNode);
						newNode.setParent(curNode);
						return;
					} else {
						curNode = curNode.getLeftChild();
					}
				} else {
					// treeItem.key >= curNode.key ------> go right
					if (curNode.getRightChild() == null) {
						curNode.setRightChild(newNode);
						newNode.setParent(curNode);
						return;
					} else {
						curNode = curNode.getRightChild();
					}
				}
			}
		}
	}
	
	private TreeItem<K, V> findItem(K key) {
		if (this.root == null) {
			return null;
		} else {
			TreeNode<K, V> curNode = this.root;
			while (curNode != null) {
				if (this.compareKeys(curNode.getTreeItem().getKey(), key) == 0) {
					return curNode.getTreeItem();
				} else if (this.compareKeys(curNode.getTreeItem().getKey(), key) < 0) {
					curNode = curNode.getRightChild();
				} else {
					curNode = curNode.getLeftChild();
				}
			}
			return null;
		}
	}
	
	private TreeNode<K, V> findNode(K key) {
		if (this.root == null) {
			return null;
		} else {
			TreeNode<K, V> curNode = this.root;
			while (curNode != null) {
				if (this.compareKeys(curNode.getTreeItem().getKey(), key) == 0) {
					return curNode;
				} else if (this.compareKeys(curNode.getTreeItem().getKey(), key) < 0) {
					curNode = curNode.getRightChild();
				} else {
					curNode = curNode.getLeftChild();
				}
			}
			return null;
		}
	}

	private boolean isLeafNode(TreeNode<K, V> node) {
		return ((node.getLeftChild() == null) && (node.getRightChild() == null));
	}

	private boolean hasRightChild(TreeNode<K, V> node) {
		return (node.getRightChild() != null);
	}

	private boolean hasNoRightChild(TreeNode<K, V> node) {
		return (node.getRightChild() == null);
	}

	private boolean hasLeftChild(TreeNode<K, V> node) {
		return (node.getLeftChild() != null);
	}

	private boolean hasNoLeftChild(TreeNode<K, V> node) {
		return (node.getLeftChild() == null);
	}

	private boolean hasNoParent(TreeNode<K, V> node) {
		return (node.getParent() == null);
	}
	
	private boolean sameNode(TreeNode<K, V> node1, TreeNode<K, V> node2) {
		return (node1.equals(node2));
	}
	
	private void deleteNode(TreeNode<K, V> nodeToDelete) {
		if (isLeafNode(nodeToDelete)) {
			// removing a leaf node
			TreeNode<K, V> parentNode = nodeToDelete.getParent();
			if (parentNode == null) {
				// removing root of tree that had no other nodes, it is a leaf and a root
				this.root = null;
				return;
			}  else {
				if ((parentNode.getLeftChild() != null) && (this.sameNode(nodeToDelete, parentNode.getLeftChild()))) {
					// removing the left leaf child of parent node
					parentNode.setLeftChild(null);
					return;
				} else {
					// removing the right leaf child of parent node
					parentNode.setRightChild(null);
				}
			}
		} else if ((this.hasLeftChild(nodeToDelete)) && (this.hasNoRightChild(nodeToDelete))) {
			// node has left child only
			// promote the left child
			TreeNode<K, V> parentNode = nodeToDelete.getParent();
			nodeToDelete.getLeftChild().setParent(parentNode);
			if (this.hasNoParent(nodeToDelete)) {
				this.root = nodeToDelete.getLeftChild();
				this.root.setParent(null);
			} else if ((hasLeftChild(parentNode)) && (sameNode(nodeToDelete, parentNode.getLeftChild()))) {
				parentNode.setLeftChild(nodeToDelete.getLeftChild());
			} else {
				parentNode.setRightChild(nodeToDelete.getLeftChild());
			}
		} else if ((this.hasRightChild(nodeToDelete)) && (this.hasNoLeftChild(nodeToDelete))) {
			// node has right child only
			// promote the right child
			TreeNode<K, V> parentNode = nodeToDelete.getParent();
			nodeToDelete.getRightChild().setParent(parentNode);
			if (this.hasNoParent(nodeToDelete)) {
				this.root = nodeToDelete.getLeftChild();
				this.root.setParent(null);
			} else if ((hasLeftChild(parentNode)) && (sameNode(nodeToDelete, parentNode.getLeftChild()))) {
				parentNode.setLeftChild(nodeToDelete.getRightChild());
			} else {
				parentNode.setRightChild(nodeToDelete.getRightChild());
			}
		} else {
			// removing  a node with two children
			TreeNode<K, V> successor = this.findSuccessor(nodeToDelete);
			nodeToDelete.setTreeItem(successor.getTreeItem());
			this.deleteNode(successor);
		}
	}
	
	private TreeNode<K, V> findSuccessor(TreeNode<K, V> node) {
		TreeNode<K, V> curNode = node.getRightChild();
		while (curNode.getLeftChild() != null) {
			curNode = curNode.getLeftChild();
		}
		return curNode;
	}
	
	
	private int compareKeys(K k1, K k2) {
		// k1 > k2 returns positive number
		// k1 == k2 returns 0
		// k1 < k2 return negative number
		return k1.compareTo(k2);
	}
	
	
	
	
	
	
	
	
	
}
