import java.nio.channels.ShutdownChannelGroupException;
import java.util.Random;

public class Driver {

	public static final int NUMBER_OF_PEOPLE = 100;
	
	private static Person<Integer>[] makePeople(int count) {
		Random rand = new Random();
		Person<Integer>[] people = new Person[count];
		
		for ( int i = 0 ; i < count ; i++ ) {
			people[i] = new Person<Integer>(new Integer(rand.nextInt(NUMBER_OF_PEOPLE * 10_000)));
		}
		
		return people;
	}
	
	public static void traverse(BinarySearchTree<Integer, Person<Integer>> tree) {
		TreeIterator<Integer, Person<Integer>> iterator = (TreeIterator<Integer, Person<Integer>>)tree.iterator();
		
		iterator.setPreOrder();
		System.out.println("\n\nPreOrder");
		showIterator(iterator);
		
		iterator.setInOrder();
		System.out.println("\n\nInOrder");
		showIterator(iterator);
		
		iterator.setPostOrder();
		System.out.println("\n\nPostsOrder");
		showIterator(iterator);
	}
	
	public static void showIterator(TreeIterator<Integer, Person<Integer>> iterator) {
		Person<Integer> person = null;
		int num = 1;
		while (iterator.hasNext()) {
			person = iterator.next().getValue();
			System.out.println(num++ + " - " + person);
		}
	}
	
	
	public static void main(String[] args) {
		Person<Integer>[] people = makePeople(NUMBER_OF_PEOPLE);

		BinarySearchTree<Integer, Person<Integer>> bst = new BinarySearchTree<Integer, Person<Integer>>();
		
		for ( int i = 0 ; i < people.length ; i++ ) {
			TreeItem<Integer, Person<Integer>> treeItem = new TreeItem<Integer, Person<Integer>>(people[i].getKey(), people[i]);
			bst.insert(treeItem);
		}
		
		traverse(bst);
	
		for ( int i = 0 ; i < people.length ; i++ ) {
			System.out.println("\n\n\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
			System.out.println("Deleting Person with key = " + people[i].getKey() + "\n\n");
			bst.delete(people[i].getKey());
			System.out.println("Tree with " + (NUMBER_OF_PEOPLE - 1 - i) + " Person Entries");
			traverse(bst);
		}
	}

}
